"use strict";

exports.__esModule = true;
exports.connectLocal = exports.onConstruct = exports.getActionCreators = exports.createShouldComponentUpdate = exports.createGetState = exports.createDispatch = void 0;

var _fastEquals = require("fast-equals");

var _react = require("react");

var _constants = require("./constants");

var _utils = require("./utils");

// external dependencies
// constants
// utils

/**
 * @function createDispatch
 *
 * @description
 * create the dispatch method for the component
 *
 * @param {ReactComponent} instance the component instance
 * @param {function} reducer the reducer method
 * @returns {function(Object): void} the dispatch method for the component
 */
var createDispatch = function createDispatch(instance, reducer) {
  var dispatch = function dispatch(action) {
    instance.__state = reducer(instance.__state, action);
    instance.setState(function () {
      return instance.__state;
    });
    return action;
  };

  var middlewareApi = {
    dispatch: dispatch,
    getState: instance.getState
  };
  return (0, _utils.composeMiddleware)(instance.options.middlewares.map(function (middleware) {
    return middleware(middlewareApi);
  }))(dispatch);
};

exports.createDispatch = createDispatch;

var createGetState = function createGetState(instance) {
  return (
    /**
     * @function getState
     *
     * @description
     * get the current state of the component
     *
     * @returns {Object} the current state
     */
    function () {
      return instance.__state;
    }
  );
};

exports.createGetState = createGetState;

var createShouldComponentUpdate = function createShouldComponentUpdate(instance) {
  return (
    /**
     * @function shouldComponentUpdate
     *
     * @description
     * determine whether the component should update based on the options provided
     *
     * @param {Object} nextProps the next props of the component
     * @param {Object} nextState the next reducer values
     * @param {Object} nextContext the next context of the component
     * @returns {boolean} should the component update
     */
    function (nextProps, nextState, nextContext) {
      var actionCreators = instance.actionCreators,
          context = instance.context,
          _instance$options = instance.options,
          areMergedPropsEqual = _instance$options.areMergedPropsEqual,
          areOwnPropsEqual = _instance$options.areOwnPropsEqual,
          areStatesEqual = _instance$options.areStatesEqual,
          mergeProps = _instance$options.mergeProps,
          pure = _instance$options.pure,
          props = instance.props,
          state = instance.state;

      if (!pure) {
        return true;
      }

      return !areOwnPropsEqual(props, nextProps) || !areStatesEqual(state, nextState) || !(0, _fastEquals.shallowEqual)(context, nextContext) || !areMergedPropsEqual( // eslint workaround
      mergeProps(state, actionCreators, props), mergeProps(nextState, actionCreators, nextProps));
    }
  );
};
/**
 * @function getActionCreators
 *
 * @description
 * get the actionCreators wrapped with the dispatch method
 *
 * @param {Object} actionCreators the actionCreators to wrap
 * @param {function} dispatch the instance-specific dispatch function
 * @returns {Object} the wrapped actionCreators
 */


exports.createShouldComponentUpdate = createShouldComponentUpdate;

var getActionCreators = function getActionCreators(actionCreators, dispatch) {
  return (0, _utils.reduce)(Object.keys(actionCreators), function (wrappedActionCreators, name) {
    wrappedActionCreators[name] = function () {
      return dispatch(actionCreators[name].apply(actionCreators, arguments));
    };

    return wrappedActionCreators;
  }, {});
};
/**
 * @function onConstruct
 *
 * @description
 * on construction of the instance, set the options and the instance-specific methods
 *
 * @param {ReactComponent} instance the component instance
 * @param {function} reducer the reducer with state
 * @param {Object} actionCreators the actionCreators to wrap with dispatch
 * @param {Object} options additional options for the component
 */


exports.getActionCreators = getActionCreators;

var onConstruct = function onConstruct(instance, reducer, actionCreators, options) {
  // state
  instance.state = reducer(undefined, {});
  instance.__state = instance.state; // options

  instance.options = (0, _utils.assign)({}, _constants.DEFAULT_OPTIONS, options); // lifecycle methods

  instance.shouldComponentUpdate = createShouldComponentUpdate(instance); // instance methods

  instance.getState = createGetState(instance);
  instance.dispatch = createDispatch(instance, reducer); // instance values

  instance.actionCreators = actionCreators ? getActionCreators(actionCreators, instance.dispatch) : {
    dispatch: instance.dispatch
  };
};
/**
 * @function connectLocal
 *
 * @description
 * decorate a component with a higher-order component that manages local state
 * in the way that redux manages global state
 *
 * @param {function} reducer the reducer with state
 * @param {Object} actionCreators the actionCreators to wrap with dispatch
 * @param {Object} options additional options for the component
 * @returns {function}
 */


exports.onConstruct = onConstruct;

var connectLocal = function connectLocal(reducer, actionCreators, options) {
  return (
    /**
     * @function ConnectedLocalComponent
     *
     * @description
     * higher-order component decorator which adds a redux-like manager for local state
     *
     * @param {ReactComponent} ComponentToConnect the component being wrapped
     * @returns {ReactComponent} the higher-order component
     */
    function (ComponentToConnect) {
      function ConnectedLocalComponent(initialProps) {
        _react.Component.call(this, initialProps);

        onConstruct(this, reducer, actionCreators, options);

        this.render = function () {
          return (0, _react.createElement)(ComponentToConnect, this.options.mergeProps(this.state, this.actionCreators, this.props));
        };

        return this;
      }

      ConnectedLocalComponent.prototype = Object.create(_react.Component.prototype);
      ConnectedLocalComponent.displayName = "ConnectedLocal(" + (0, _utils.getDisplayName)(ComponentToConnect) + ")";
      return ConnectedLocalComponent;
    }
  );
};

exports.connectLocal = connectLocal;